
# Metarace : Cycle Race Abstractions
# Copyright (C) 2012-15  Nathan Fraser
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Yocto-Meteo Interface."""
import threading
import logging
import time	# for sleep in main thread
from yocto_api import YAPI
from yocto_humidity import YHumidity
from yocto_temperature import YTemperature
from yocto_pressure import YPressure

class meteo(threading.Thread):
    """Yocto-Meteo Object"""
    def __init__(self):
        threading.Thread.__init__(self) 
        self.__m = None
        self.__t = None
        self.__t = None
        self.__p = None
        self.t = None
        self.h = None
        self.p = None
        self.__stat = False
        self.__running = False
        self.log = logging.getLogger(u'meteo')
        self.log.setLevel(logging.DEBUG)

    def connected(self):
        return self.__stat

    def exit(self):
        """Request thread termination."""
        self.__running = False

    def run(self):
        """Called via threading.Thread.start()."""
        self.__running = True
        self.log.debug('Starting')
        YAPI.RegisterHub("usb")
        while self.__running:
            try:
                if self.__m is not None and self.__m.isOnline():
                    #    fetch params
                    self.t = self.__t.get_currentValue()
                    self.h = self.__h.get_currentValue()
                    self.p = self.__p.get_currentValue()
                    #    set connection status
                    self.__stat = True
                else:
                    #self.log.debug(u'Meteo offline.')
                    self.__stat = False
                    self.__m = None	# invalidate the old connections
                    self.__t = None
                    self.__h = None
                    self.__p = None
                    self.t = None
                    self.h = None
                    self.p = None
                    YAPI.UpdateDeviceList()
                    sensor = YHumidity.FirstHumidity()
                    if sensor is None:
                        self.log.debug(u'No meteo detected.')
                    else:
                        self.__m = sensor.get_module()
                        target = self.__m.get_serialNumber()
                        self.log.debug(u'Meteo re-connect: ' + repr(target))
                        self.__t = YTemperature.FindTemperature(
                                          target+'.temperature')
                        self.__h = YHumidity.FindHumidity(
                                          target+'.humidity')
                        self.__p = YPressure.FindPressure(
                                          target+'.pressure')
                time.sleep(10)
            except Exception as e:
                self.log.error('Exception: ' + str(type(e)) + str(e))
                raise
        self.log.info('Exiting')

def printval(h=None):
    if h.connected():
        print(u'{0:0.2f}C {1:0.0f}% {2:0.0f}mb'.format(h.t, h.h, h.p))
    else:
        pass
    return True

if __name__ == "__main__":
    import metarace
    import gtk
    import glib
    import time
    metarace.init()
    y = meteo()
    lh = logging.StreamHandler()
    lh.setLevel(logging.DEBUG)
    y.log.addHandler(lh)
    try:
        y.start()
        glib.timeout_add_seconds(15, printval, y)	# in main loop?
        metarace.mainloop()
    except:
        y.exit()
        y.join()
        raise
